// C++ code
//
// Cobbled together from previous Tinkercad submissions as well as parts
// of 9/16's lab code (Servos) and 9/22's lab code (Ultrasonic Ping).
//
// Ultrasonic Code is NOT my work, attributed to David A. Mellis and Tom Igoe
// as part of public domain at https://www.arduino.cc/en/Tutorial/BuiltInExamples/Ping.
//
// Remaining code written by Marcus Troy on 10/8/21, with libraries imported as necessary.

#include <Servo.h>   // Load Servo Library
Servo myservoS;      // Create servo object (positional, holds ultrasonic sensor).
Servo myservoL;      // Create servo object (rotational continuous, left side).
Servo myservoR;      // Create servo object (rotational continuous, right side).
int angle = 90;      // Declare variable for initial positional servo angle (set to 90 as motor is centered in range of motion of 180 deg.
int dt = 10;         // Define a short delay time.
int loopTrack = 5;   // Define a longer loop time that will be used later on for the bot's movement loop. (REDUCED AFTER CIRCUIT TESTING)
int loopCurrent = 0; // Defines variable to work with looptrack

//const int pingPin = 12;  // NEEDED TO BE CHANGED FROM ORIGINAL EXAMPLE CODE, PING NOW SPLIT INTO ECHO AND TRIGGER PINS FOR HC-SR04 ULTRASONIC SENSOR
#define trigPin 13       // Syntax copied from www.instructables.com/Simple-Arduino-and-HC-SR04-Example/
#define echoPin 12       // The Trigger pin is the pin that RECIEVES the feedback from the "echo" sent by the Echo pin

void setup()
{
  Serial.begin(9600);         // Initialize serial communication with Ultrasonic Sensor.
  myservoR.attach(9);        // Tell the Arduino which pin (11) will drive the servo (Right motor).
  myservoS.attach(10);        // Tell the Arduino which pin (10) will drive the servo (Center motor).
  myservoL.attach(11);         // Tell the Arduino which pin (9) will drive the servo  (Left motor).
  pinMode(trigPin, OUTPUT);   // Establishes trigger pin (again, syntax copied).
  pinMode(echoPin, INPUT);    // Establishes echo pin.
  
}

void loop()
{
  long duration, inches, cm;   // Establish variables for duration of the ping, leads to the distance
                               // result appearing in inches and centimeters:
  
  //pinMode(pingPin, OUTPUT);  // The PING is triggered by a HIGH pulse of 2 or more microseconds. (NOT NEEDED DUE TO SENSOR DIFFERENCES)
  digitalWrite(trigPin, LOW);  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  delayMicroseconds(2);        // Matches the example code from Instructables surpisingly.
  digitalWrite(trigPin, HIGH); 
  delayMicroseconds(10);       // Delay changed from 5ms to 10ms, may or may not be relevant based on differences in the lab ultrasonic sensor and the HC-SR04
  digitalWrite(trigPin, LOW);
  
  //pinMode(pingPin, INPUT);          // Old code, new code used above
  duration = pulseIn(echoPin, HIGH);  // whose duration is the time (in microseconds) from the sending of the ping
                                      // to the reception of its echo off of an object. NOW PROPERLY USES ECHOPIN.
    
  cm = (duration/2)/29.1;  // Convert the time into a distance. ADJUSTED FROM INSTRUCTABLES CODE, SHIFTED CONVERSION INTO LOOP AS OPPOSED TO OUT OF LOOP
  
  Serial.print(cm);      // Use for troubleshooting purposes in actual project:
  Serial.println(" cm");
  //Serial.println();
  
  if(cm<10)    // First in functional series of checks using the ultrasonic sensor, this checks to see if there is a barrier DIRECTLY IN FRONT of the robot and executes if it does
  {
    while(angle<180)
    {
      angle++;                // Slowly increment angle measure variable (in CW direction: to RIGHT of robot)
      myservoS.write(angle);  // Tell servo to move to newly defined angle
      delay(dt);              // Slight delay now used (in ms), can be adjusted for performance
    }
    if(cm<5)   // Second ultrasonic check, if path in front of robot is blocked, check to the RIGHT of the robot after sensor is rotated to that direction.
    {
      while(angle>0)  // If the second check does NOT pass, start looking in the other direction
      {
        angle--;                // Slowly increment angle measure variable (in CCW direction, to LEFT of robot:)
        myservoS.write(angle);  // Tell servo to move to newly defined angle
        delay(dt);              // Slight delay now used (in ms), can be adjusted for performance
      }
    }
    else        // If the second check passes, fire the servos on the LEFT side of the robot forwards in hopes of rotating the design RIGHT +90deg (WILL NEED TO BE TUNED HEAVILY)
    {
      while(loopCurrent<loopTrack)
      {
        myservoL.write(0); // Write function operates differently based on continuous servo syntax, see https://www.arduino.cc/en/Reference/ServoWrite for details.
        loopCurrent++;     // Functional timer
      }
      loopCurrent = 0;     // Resets loopCurrent timer for future loops.
      myservoL.write(90);  // Brings left side continuous servo to stop.
    }        
    if(cm<10)    // Third and final ultrasonic check, if path in front of robot is STILL blocked, then reset the ultrasonic orientation to forwards facing and fire both leg servos in reverse.
    {
      while(angle<90)  // If the third check does NOT pass, reset ultrasonic sensor to initial position
      {
        angle++;                // Slowly increment angle measure variable (in CCW direction, to LEFT of robot:)
        myservoS.write(angle);  // Tell servo to move to newly defined angle
        delay(dt);              // Slight delay now used (in ms), can be adjusted for performance
      }
      while(loopCurrent<loopTrack) // Now send the robot in full reverse to back out of whatever corner it has found itself in.
      {
        myservoL.write(45);   // Fire both motors in opposite directions to both each other (servos are flipped in two directions, need to be synced for straightforward motion)
        myservoR.write(135);  // as well as set motor speeds to the opposite of when they are to fire in forwards direction.
        loopCurrent++;        // Functional timer
      }
      loopCurrent = 0;        // Resets loopCurrent timer for future loops.
      myservoL.write(90);     // Brings left side continuous servo to stop.
      while(loopCurrent<loopTrack)  // So that the robot isn't doomed to repeat entering and leaving a corner, attempt to turn and pathfind once unstuck a little.
      {
        myservoL.write(0); // Write function operates differently based on continuous servo syntax, see https://www.arduino.cc/en/Reference/ServoWrite for details.
        loopCurrent++;     // Functional timer
      }
      loopCurrent = 0;     // Resets loopCurrent timer for future loops.
      myservoL.write(90);  // Brings left side continuous servo to stop.
    }
    else         // If the third check passes, fire the servos on the RIGHT side of the robot forwards in hopes of rotating the design LEFT -90deg
    {
      while(loopCurrent<loopTrack)
      {
        myservoR.write(180);  // Same function as rotating robot with while loop earlier, just using other servo.
        loopCurrent++;        // Functional timer
      }
      loopCurrent = 0;        // Resets loopCurrent timer for future loops.
      myservoL.write(90);     // Brings left side continuous servo to stop.
    }
  }
  else           // Congratulations! The robot is, at least for this loop, NOT IMPEDED. Full speed ahead!
  {
    while(loopCurrent<loopTrack)
      {
        myservoL.write(45);    // Fire both motors in opposite directions to each other (servos are flipped in two directions, need to be synced for straightforward motion)
        myservoR.write(135);  // as well as set motor speeds to the same as when they are to fire in forwards direction for rotation.
        loopCurrent++;        // Functional timer
      }
      loopCurrent = 0;        // Resets loopCurrent timer for future loops.
      //myservoL.write(90);     // Brings left side continuous servo to stop.
   }
}
  
// This code does not have an end function as it is designed with the mechanical slide switch in mind. Once the arduino is turned on and the battery is active, the only way to stop the
// system is to disconnect the circuit by sliding the slide switch downwards. Or to turn off the battery. Either works.

// Second 
